% Main program to run MC simulations
% Written by: Cynthia Yang
% Last updated: January 2020
clear all;
close all;
date_string = '122019';  % date to be included in output filenames
fdir ='.'; % directory (optional)
%==========================================================================
% Change the following values to run different simulations
flag_NormalErr = 0; % =1: Gaussian errors; =0: Chi2 errors
flag_HetVar = 0;  % =1: heteroskedasticity; =0: homoskedasticity
flag_SARX = 1;  % =1: SARX model; =2: SAR model without X
flag_Wx = 0;    % =1: Wx is different from Wy; =0: Wx = Wy
flag_expodecay = 0; % =1: W with exponentially decaying delta's; =0: finite number of positive delta's
if flag_expodecay == 0
    deltaset = [0,0.25,0.5,0.75,0.95,1]; % possible values of delta
    num_domi = 1;   % number of dominant units
elseif flag_expodecay == 1
    q_decay = 0.9;
    deltaset = [0]; % this is a dummy and not used
end        
% rhoset = [0.2,0.5,0.75,0.95]; % possible values of rho
rhoset = [0.2,0.5,0.75];  % For SARX models, rho cannot be 0.95
Nset = [100, 300, 500, 1000]; % possible values of sample size
nsimul = 2000;   % number of replications
option = 3.1;   % Estimation Option: set to different values to run different estimations
              % MLE=1
              % BMM assuming homoskedasticity=3; BMM assuming heteroskedasticity=3.1
              % GMM/OGMM/BGMM assuming homoskedasticity=4, 4.1, 4.2, respectively
              % GMM/OGMM/BGMM assuming heteroskedasticity=5, 5.1, 5.2, respectively
              % Note: OGMM and BGMM load GMM results as first-step estimates to improve speed
%==========================================================================
T = 1;        % cross section model; T>1 is left for future extensions
alpha = 1;
beta = 1;
deltax = 1;   % delta for Wx
if flag_SARX==1
    model_string = 'SARX';
    lambda = 0.75;   
    betavec = [alpha;beta];
elseif flag_SARX==0
    model_string = 'SAR';
    betavec = alpha;
end
if nsimul==2
    RsqBIG = zeros(length(deltaset),length(Nset),length(rhoset));
    if flag_SARX==1
        sigv2BIG = zeros(length(deltaset),length(Nset),length(rhoset));
    end
end
sige = 1;
sige2 = sige^2;
if flag_NormalErr==1
    err_string = 'normal';
elseif flag_NormalErr==0
    err_string = 'chi2';
end
if flag_HetVar==1
    err_string = [err_string,'_HetVar'];
end
%==========================================================================
for rhoi = 1:length(rhoset)   % loop across rho
    rho = rhoset(rhoi);
for n = 1:length(Nset)        % loop across sample size
    for deltai = 1:length(deltaset)  % loop across delta
        N = Nset(n);
        delta = deltaset(deltai);

if flag_expodecay==0
    N1 = min(N-1,floor(N^delta));
    Wini = zeros(N,N);
    rng(6000);
    if option==1 && N==1000 && delta==0
        rng(6001);  % use another seed to avoid MLE numerical coincidence
    end
    domi_weights = rand(N1,num_domi);
    domi_weights0 = [zeros(1,num_domi);domi_weights];
    for j = 1:num_domi
        Wini(1:N1+1,j) = circshift(domi_weights0(:,j),j-1);  % make sure Wii=0
    end
    Wini(1:num_domi,num_domi+1:9+num_domi-1)= 1;
    Wini(num_domi+1:N,num_domi+1:N) = qqWeight(N-num_domi,4);
    W = normw(Wini); 
elseif flag_expodecay==1
    deltaN = q_decay.^(1:N);
    Nnz_vec = floor(N.^deltaN);
    Wini = zeros(N,N);
    for j = 1:N-1
        Wini(j+1:j+Nnz_vec(j),j)=1;
    end
    Wini(1,N) = 1;
    rng(6001);
    weights = rand(N,N);
    W = normw(Wini.*weights);
    if min(sum(W,2))<=0
        error('row sums error!');
    end
end
    
S = speye(N) - rho*W;
invS = inv(S);
if flag_SARX==1
    if flag_Wx==0
        Wx = W;  
    elseif flag_Wx==1
        N1x = min(N-1,floor(N^deltax));
        Wxini = zeros(N,N);
        rng(8000);
        domix_weights = rand(N1x,num_domi);
        domix_weights0 = [zeros(1,num_domi);domix_weights];
        for j = 1:num_domi
            Wxini(1:N1x+1,j) = circshift(domix_weights0(:,j),j-1);
        end
        Wxini(1:num_domi,num_domi+1:9+num_domi-1)= 1;
        Wxini(num_domi+1:N,num_domi+1:N) = qqWeight(N-num_domi,4);
        Wx = normw(Wxini); 
    end
    Sx = speye(N) - lambda*Wx;
    invSx = inv(Sx);
    if flag_HetVar==0
        SSrho = S'*S;
        SSlambda = Sx'*Sx;
        invSSrho = inv(SSrho);
        invSSlambda = inv(SSlambda);
        trSxy = trace(invSSlambda*invSSrho');
        trSy = trace(invSSrho);
        an = trSxy/trSy;   
        Rsq0 = 1 - N/trace(invSSrho);
        sigv2 = 0.1/(0.9-Rsq0)/an*sige2/beta^2;
        if sigv2 <=0
            error('  negative sigv2!');
        end
        if Rsq0>=0.9
            error('  Rsq0 exceeds 0.9!');
        end
        sigv = sigv2^0.5;
        % compute analytical Rsq    % not necessary
        trVary = beta^2*sigv2*trSxy + sige2*trSy;
        Rsq = 1 - N*sige2/trVary;
    elseif flag_HetVar==1
        rng(10000);
        tau2_mtx = 0.5 + (1.5-0.5).*rand(N,2);
        tau2_e = tau2_mtx(:,1);
        tau2_v = tau2_mtx(:,2);
        tau_e = tau2_e.^0.5;
        tau_v = tau2_v.^0.5;
        trDe = sum(tau2_e);
        trSyD = trace(invS.*tau2_e'*invS');
        invSSx = invS*invSx;
        trSxyD = trace(invSSx.*tau2_v'*invSSx');
        an = trSxyD/trSyD;
        Rsq0 = 1 - trDe/trSyD;
        sigv2 = 0.1/(0.9-Rsq0)/an*sige2/beta^2;
        if sigv2 <=0
            error('  negative sigv2!');
        end
        if Rsq0>=0.9
            error('  Rsq0 exceeds 0.9!');
        end
        sigv = sigv2^0.5;        
        trVary = beta^2*sigv2*trSxyD + sige2*trSyD;
        Rsq = 1 - trDe/trVary;          
    end    
    if Rsq<=0 ||Rsq>=1
       error('  Rsq exceeds bounds!'); 
    end    
    
elseif flag_SARX==0
    Rsq = 1 - N/trace(invS*invS');  % evaluated at true rho   
end
if nsimul==2  % store the results if nsimul=2 (run nsimul=2 to compute Rsq)
    RsqBIG(deltai,n,rhoi) = Rsq;
    if flag_SARX==1
        sigv2BIG(deltai,n,rhoi) = sigv2;
    end
end

k = size(betavec,1);  % number of exogeneous regressors (including intercept)
if option==3||round(option)==4
    k1=0; 
else
    k1=1;    % do not estimate error variance
end
parmhatBIG = zeros(nsimul,k+2-k1);
seBIG = zeros(size(parmhatBIG));
mark_boundary = zeros(nsimul,1);
exitflagBIG = 99*ones(nsimul,1);

if flag_expodecay==0
    if option==4.1||option==4.2||option==5.1||option==5.2 
        % load GMM results as first-step estimates for OGMM and BGMM estimation
        option_load = round(option); 
        if flag_SARX==1
            filepath = [fdir,'\results_SARX_',date_string,'\'];
            filename =  [filepath,'results_',date_string,'_',model_string,'_',err_string,...
                '_',num2str(option_load),'_',num2str(rho),'_',num2str(N),'_',num2str(delta),'.mat'];
        elseif flag_SARX==0
            filepath = [fdir,'\results_SAR_',date_string,'\'];
            filename =  [filepath,'results_',date_string,'_',model_string,'_',err_string,...
                '_',num2str(option_load),'_',num2str(rho),'_',num2str(N),'_',num2str(delta),'.mat'];
        end
        load(filename, 'parmhatBIG');
        parmhatBIG_GMM = parmhatBIG; 
        parmhatBIG = zeros(size(parmhatBIG_GMM));
        load(filename, 'seBIG');
        seBIG_GMM = seBIG;
        seBIG = zeros(size(parmhatBIG_GMM));
    end
end
W2 = W*W;
W20 = W2 - (1/N)*trace(W2)*eye(N);
Mn = eye(N)-(ones(N,1)*ones(1,N))./N;
W20_diag = W2 - diag(diag(W2));

seed = 30*(1:nsimul);  
for iter = 1: nsimul  
    if mod(iter,50)==0
        disp(iter);
    end
%% DGP
    si = seed(iter);
    rng(si);
    if flag_NormalErr==1
        eNT = randn(N,T);
        si = si+1;
        if flag_SARX==1
            rng(si);
            vNT = randn(N,T);
        end
    elseif flag_NormalErr==0
        eNT = (chi2rnd(2,[N,T])-2)*0.5;
        si = si+1;
        if flag_SARX==1
            rng(si);
            vNT = (chi2rnd(2,[N,T])-2)*0.5;
        end
    end
    if flag_HetVar==0
        uNT = invS*eNT*sige;
        U = uNT(:);
    elseif flag_HetVar==1
        uNT = invS*(tau_e.*eNT)*sige;
        U = uNT(:);
    end    
    if flag_SARX==1
        if flag_HetVar==0
            xNT = invSx*vNT*sigv;
        elseif flag_HetVar==1
            xNT = invSx*(tau_v.*vNT)*sigv;
        end           
        yNT = invS*(beta*xNT + alpha) + uNT;
        X = xNT(:);
    elseif flag_SARX==0
        yNT = invS*(alpha*ones(N,T)) + uNT;
        X = [];
    end
    Y = yNT(:);
    ystarNT = W*yNT;
    Ystar = ystarNT(:);
%% MLE 
if option==1
    X_winpt = [ones(N,1),X];     % including inpt
    info.lflag = 0;  % no approximation for lndet
    results_MLE = sar2(Y,X_winpt,W,info);
    rhohat = results_MLE.rho;
    betavechat = results_MLE.beta;
    sige2hat = results_MLE.sige;
    se_rhohat = results_MLE.pstd;
    se_betavechat = results_MLE.bstd;
    se_sige2hat = results_MLE.sesige;
    
    eps_bound = 1e-3;
    if abs(rhohat - (-1))<eps_bound
        mark_boundary(iter)= -1;
        rhohat = -1+eps_bound;
    elseif abs(rhohat - 1)<eps_bound
        mark_boundary(iter)= 1;
        rhohat = 1-eps_bound;
    end    
    parmhatBIG(iter,1) = rhohat;
    parmhatBIG(iter,2:k+1) = betavechat;
    parmhatBIG(iter,k+2) = sige2hat;   
    seBIG(iter,1) = se_rhohat;
    seBIG(iter,2:k+1) = se_betavechat;
    seBIG(iter,k+2) = se_sige2hat;   
end
%% BMM
if round(option)==3   
    clear options
    X_winpt = [ones(N,1),X]; 
    Mb = 1000;   
    if option==3  % assuming homoskedasticity
        parm0 = [0;zeros(k,1);1]; 
        parm_lb = [-1;-Mb*ones(k,1);0];
        parm_ub = [1;Mb*ones(k+1,1)];
    elseif option==3.1 % assuming heteroskedasticity
        parm0 = [0;zeros(k,1)]; 
        parm_lb = [-1;-Mb*ones(k,1)]; 
        parm_ub = [1;Mb*ones(k,1)];        
    end
    options = optimoptions(@lsqnonlin,'Algorithm','trust-region-reflective','Display','off');
%     options.Display = 'iter';
%     options.Display = 'iter-detailed';
    options.MaxFunEvals = 1e6;
    options.MaxIter = 1e6;
    if option==3
        [parmhat,resnorm,residual,exitflag,output] = ...
            lsqnonlin(@BMM_moment_SARX_iid,parm0,parm_lb,parm_ub,options,Y,X_winpt,Ystar,W,N);
        sige2hat = parmhat(k+2);
        if sige2hat<=0
            error('negative sige2hat!');
        end
        parmhatBIG(iter,k+2) = sige2hat;   
    elseif option==3.1
        [parmhat,resnorm,residual,exitflag,output] = ...
            lsqnonlin(@BMM_moment_SARX_het,parm0,parm_lb,parm_ub,options,Y,X_winpt,Ystar,W,N);
    end
    exitflagBIG(iter,1) = exitflag;
    if exitflag~=1 && exitflag~=3
        fprintf('\n iter=%d exitflag=%d \n',iter,exitflag);
        pause;
    end
    rhohat = parmhat(1);
    betavechat = parmhat(2:k+1);
    eps_bound = 1e-3;
    if abs(rhohat-parm_lb(1))<eps_bound
       mark_boundary(iter)= -1;
       rhohat = -1+eps_bound;   
    elseif abs(rhohat-parm_ub(1))<eps_bound 
       mark_boundary(iter)= 1;
       rhohat = 1-eps_bound;    
    end   
    parmhatBIG(iter,1) = rhohat;
    parmhatBIG(iter,2:k+1) = betavechat;
    
    % Variance
    Shat = speye(N)-rhohat*W;
    Ghat = W/Shat;
    GXb = Ghat * X_winpt *betavechat;
    GXb2 = GXb'*GXb/N;
    ehatN = Y - rhohat*Ystar - X_winpt*betavechat; 
    if option==3   % assuming homoskedasticity 
                   % didn't code the variance of sig2         
        Mx = eye(N) - X_winpt/(X_winpt'*X_winpt)*X_winpt';
        % H matrix
        H = zeros(k+1,k+1);
        trGhat = trace(Ghat);
        Gterm = sige2hat*(sum(sum(Ghat.*Ghat)+sum(Ghat.*Ghat'))/N ...
            - sum(sum(Ghat.*Mx))*trGhat*2/N^2);
        H(1,1) = GXb2 + Gterm;
        H(2:k+1,1) = X_winpt'*GXb/N;
        H(1,2:k+1) = H(2:k+1,1)';
        Sigxx =  X_winpt'*X_winpt/N;
        H(2:k+1,2:k+1) = Sigxx;
        % Var(g) matrix
        C = zeros(k+1,k+1);
        C(2:k+1,1) = sige2hat*H(2:k+1,1);
        C(1,2:k+1) = C(2:k+1,1)';
        C(2:k+1,2:k+1) = sige2hat*Sigxx;
        D = Ghat - Mx*(trGhat/N);
        Dterm = sige2hat^2*sum(sum(D.*D)+sum(D.*D'))/N;
        gamma2hat = sum(ehatN.^4)/N - 3*sige2hat^2; 
        diagD = diag(D);
        non_normal_term1 = gamma2hat*sum(diagD.*diagD)/N;
        mu3 = sum(ehatN.^3)/N; 
        non_normal_term2 = 2*mu3*sum(diagD.*GXb)/N;
        non_normal_term = non_normal_term1 + non_normal_term2;  
        C(1,1) = sige2hat*GXb2 + non_normal_term + Dterm;
    elseif option==3.1  % assuming heteroskedasticity
        e2hatN = ehatN.^2;
        SigGxb = e2hatN.*GXb;
       % H matrix
        H = zeros(k+1,k+1);
        diagG = diag(Ghat);
        Gterm = sum( diag((Ghat^2 + Ghat'*Ghat - diagG.*Ghat*2)) .* e2hatN)/N;
        H(1,1) = GXb2 + Gterm;
        H(2:k+1,1) = X_winpt'*GXb/N;
        H(1,2:k+1) = H(2:k+1,1)';
        Sigxx =  X_winpt'*X_winpt/N;
        H(2:k+1,2:k+1) = Sigxx;
        % Var(g) matrix
        C = zeros(k+1,k+1);
        C(2:k+1,1) = X_winpt'*SigGxb/N;      
        C(1,2:k+1) = C(2:k+1,1)';
        C(2:k+1,2:k+1) =  X_winpt'*(e2hatN.*X_winpt)/N;              
        D = Ghat;
        D(1:N+1:end) = 0;
        SigD = e2hatN.*D;        
        Dterm = trace(SigD * (SigD + e2hatN.*D'))/N;
        C(1,1) = GXb'*SigGxb/N + Dterm;
    end
        covmtx = H\C/H/N;
        se = sqrt(diag(covmtx));
        se_rhohat = se(1);
        se_betavechat =se(2:k+1);
        seBIG(iter,1) = se_rhohat;
        seBIG(iter,2:k+1) = se_betavechat;      
end
%% GMM
if option==4 || option==5
   if flag_SARX==1 
        clear options P;
        X_winpt = [ones(N,1),X];  % including intercept
        L = [Ystar,X_winpt];      % original regressors
        WX = W*X;                 % cannot include intercept 
        W2X = W2*X;
        Q = [X_winpt,WX,W2X];     % IV
        q = size(Q,2);
        P(:,:,1) = W;
        if option==4
            P(:,:,2) = W20;
        elseif option==5
            P(:,:,2) = W20_diag;
        end
        nP = size(P,3);
        Wg(:,:,1) = eye(q+nP);        
        
        options = optimoptions(@lsqnonlin,'Algorithm','trust-region-reflective','Display','off');
        options.MaxFunEvals = 1e6;
        options.MaxIter = 1e6;
        options.TolFun = 1e-6;  
        parm0 = zeros(k+1,1);
        Mb = 1000;                  
        parm_lb = [-1;-Mb*ones(k,1)];
        parm_ub = [1;Mb*ones(k,1)];    
        [parmhat_GMM,resnorm,residual,exitflag,output] = ...
            lsqnonlin(@GMM_multiP_moment_SARX,parm0,parm_lb,parm_ub,options,Y,L,Q,P,Wg(:,:,1),N,T,0);
        exitflagBIG(iter) = exitflag;
        eps_bound = 1e-3;
        if abs(parmhat_GMM(1) - parm_lb(1))<eps_bound
           mark_boundary(iter)= -1;
           parmhat_GMM(1) = -1+eps_bound;
        elseif abs(parmhat_GMM(1)-parm_ub(1))<eps_bound
           mark_boundary(iter)= 1;
           parmhat_GMM(1) = 1-eps_bound;
        end  
        parmhatBIG(iter,1:k+1) = parmhat_GMM;
        if option==4
            [Dt,Sigmag_GMM,covmtx_GMM] = GMM_multiP_var_SARX_iid(parmhat_GMM,Y,L,Q,P,Wg(:,:,1),W,N,k,1);
        elseif option==5
            [Dt,Sigmag_GMM,covmtx_GMM] = GMM_multiP_var_SARX_het(parmhat_GMM,Y,L,Q,P,Wg(:,:,1),W,N,k,1);
        end
        seBIG(iter,1:k+1) = sqrt(diag(covmtx_GMM));
    elseif flag_SARX==0   % only has an intercept, cannot use IV
        clear options P;
        X_winpt = [ones(N,1),X];   
        L = [Ystar,X_winpt]; 
        Q = X_winpt;   
        q = size(Q,2);
        P(:,:,1) = W;
        if option==4
            P(:,:,2) = W20;
        elseif option==5
            P(:,:,2) = W20_diag;
        end
        nP = size(P,3);
        Wg(:,:,1) = eye(q+nP);        
        
        options = optimoptions(@lsqnonlin,'Algorithm','trust-region-reflective','Display','off');
        options.MaxFunEvals = 1e6;
        options.MaxIter = 1e6;
        options.TolFun = 1e-6;  
        parm0 = zeros(k+1,1);
        Mb = 1000;                  
        parm_lb = [-1;-Mb*ones(k,1)];
        parm_ub = [1;Mb*ones(k,1)];    
        [parmhat_GMM,resnorm,residual,exitflag,output] = ...
            lsqnonlin(@GMM_multiP_moment_SARX,parm0,parm_lb,parm_ub,options,Y,L,Q,P,Wg(:,:,1),N,T,0);
        exitflagBIG(iter) = exitflag;
        eps_bound = 1e-3;
        if abs(parmhat_GMM(1) - parm_lb(1))<eps_bound
           mark_boundary(iter)= -1;
           parmhat_GMM(1) = -1+eps_bound;
        elseif abs(parmhat_GMM(1)-parm_ub(1))<eps_bound
           mark_boundary(iter)= 1;
           parmhat_GMM(1) = 1-eps_bound;
        end  
        parmhatBIG(iter,1:k+1) = parmhat_GMM;
        if option==4
            [Dt,Sigmag_GMM,covmtx_GMM] = GMM_multiP_var_SARX_iid(parmhat_GMM,Y,L,Q,P,Wg(:,:,1),W,N,k,1);
        elseif option==5
            [Dt,Sigmag_GMM,covmtx_GMM] = GMM_multiP_var_SARX_het(parmhat_GMM,Y,L,Q,P,Wg(:,:,1),W,N,k,1);
        end
        seBIG(iter,1:k+1) = sqrt(diag(covmtx_GMM));
    end
end
%% OGMM (need to load GMM results)
if option==4.1 || option==5.1
    if flag_SARX==1
        clear options P;
        X_winpt = [ones(N,1),X];    
        WX = W*X;           
        L = [Ystar,X_winpt];  
        W2X = W2*X;
        Q = [X_winpt,WX,W2X]; 
        q = size(Q,2);
        P(:,:,1) = W;
       if option==4.1
            P(:,:,2) = W20;
        elseif option==5.1
            P(:,:,2) = W20_diag;
        end
        nP = size(P,3);
        Wg(:,:,1) = eye(q+nP);        
        parmini = parmhatBIG_GMM(iter,1:k+1)'; 
        if option==4.1
            [Dt_GMM,Sigmag_GMM,covmtx_GMM] = GMM_multiP_var_SARX_iid(parmini,Y,L,Q,P,Wg(:,:,1),W,N,k,1);
        elseif option==5.1
            [Dt_GMM,Sigmag_GMM,covmtx_GMM] = GMM_multiP_var_SARX_het(parmini,Y,L,Q,P,Wg(:,:,1),W,N,k,1);
        end
        Wg(:,:,2) = pinv(Sigmag_GMM);
        options = optimoptions(@lsqnonlin,'Algorithm','trust-region-reflective','Display','off');
        options.MaxFunEvals = 1e6;
        options.MaxIter = 1e6;
        options.TolFun = 1e-6;
        Mb = 1000;                  
        parm_lb = [-1;-Mb*ones(k,1)];
        parm_ub = [1;Mb*ones(k,1)];    
        [parmhat_OGMM,resnorm,residual,exitflag,output] = ...
            lsqnonlin(@GMM_multiP_moment_SARX,parmini,parm_lb,parm_ub,options,Y,L,Q,P,Wg(:,:,2),N,T,1);
        exitflagBIG(iter) = exitflag;
        eps_bound = 1e-3;
        if abs(parmhat_OGMM(1) - parm_lb(1))<eps_bound
           mark_boundary(iter)= -1;
           parmhat_OGMM(1) = -1+eps_bound;
        elseif abs(parmhat_OGMM(1)-parm_ub(1))<eps_bound
           mark_boundary(iter)= 1;
           parmhat_OGMM(1) = 1-eps_bound;
        end  
        parmhatBIG(iter,1:k+1) = parmhat_OGMM;
        if option==4.1
            [Dt_OGMM,Sigmag_OGMM,covmtx_OGMM] = GMM_multiP_var_SARX_iid(parmhat_OGMM,Y,L,Q,P,Wg(:,:,2),W,N,k,0);  
        elseif option==5.1
            [Dt_OGMM,Sigmag_OGMM,covmtx_OGMM] = GMM_multiP_var_SARX_het(parmhat_OGMM,Y,L,Q,P,Wg(:,:,2),W,N,k,0);
        end
        seBIG(iter,1:k+1) = sqrt(diag(covmtx_OGMM));   
    elseif flag_SARX==0 
        clear options P;
        X_winpt = [ones(N,1),X]; 
        
        L = [Ystar,X_winpt]; 
        Q = X_winpt;  
        q = size(Q,2);
        P(:,:,1) = W;
        if option==4.1
            P(:,:,2) = W20;
        elseif option==5.1
            P(:,:,2) = W20_diag;
        end
        nP = size(P,3);
        Wg(:,:,1) = eye(q+nP);        
        parmini = parmhatBIG_GMM(iter,1:k+1)';    
        [Dt_GMM,Sigmag_GMM,covmtx_GMM] = GMM_multiP_var_SARX_iid(parmini,Y,L,Q,P,Wg(:,:,1),W,N,k,1);
        Wg(:,:,2) = pinv(Sigmag_GMM);
        options = optimoptions(@lsqnonlin,'Algorithm','trust-region-reflective','Display','off'); 
        options.MaxFunEvals = 1e6;
        options.MaxIter = 1e6;
        options.TolFun = 1e-6;
        Mb = 1000;                  
        parm_lb = [-1;-Mb*ones(k,1)];    
        parm_ub = [1;Mb*ones(k,1)];    
        [parmhat_OGMM,resnorm,residual,exitflag,output] = ...
            lsqnonlin(@GMM_multiP_moment_SARX,parmini,parm_lb,parm_ub,options,Y,L,Q,P,Wg(:,:,2),N,T,1);
        exitflagBIG(iter) = exitflag;
        eps_bound = 1e-3;
        if abs(parmhat_OGMM(1) - parm_lb(1))<eps_bound
           mark_boundary(iter)= -1;
           parmhat_OGMM(1) = -1+eps_bound;
        elseif abs(parmhat_OGMM(1)-parm_ub(1))<eps_bound
           mark_boundary(iter)= 1;
           parmhat_OGMM(1) = 1-eps_bound;
        end  
        parmhatBIG(iter,1:k+1) = parmhat_OGMM;
        [Dt_OGMM,Sigmag_OGMM,covmtx_OGMM] = ...
            GMM_multiP_var_SARX_iid(parmhat_OGMM,Y,L,Q,P,Wg(:,:,2),W,N,k,0);
        seBIG(iter,1:k+1) = sqrt(diag(covmtx_OGMM));   
    end
end
%% BGMM (need to load GMM results)
if option==4.2 || option==5.2    
    if flag_SARX==1
        clear options;
        X_winpt = [ones(N,1),X];
        L = [Ystar,X_winpt];
        parmini = parmhatBIG_GMM(iter,1:k+1)';
        Shat = speye(N)-parmini(1)*W;
        Ghat = W/Shat;
        if option==4.2
            Phat = Ghat-trace(Ghat)/N*eye(N);
        elseif option==5.2
            Phat = Ghat - diag(diag(Ghat));    
        end
        Qhat = [X_winpt, Ghat*X_winpt*parmini(2:k+1)];    
        q = size(Qhat,2);
        Wg(:,:,1) = eye(q+1);
        options = optimoptions(@lsqnonlin,'Algorithm','trust-region-reflective','Display','off');
        options.MaxFunEvals = 1e6;
        options.MaxIter = 1e6;
        options.TolFun = 1e-6;
        Mb = 1000;                  
        parm_lb = [-1;-Mb*ones(k,1)]; 
        parm_ub = [1;Mb*ones(k,1)];    
        [parmhat_BGMM1,resnorm,residual,exitflag,output] = ...
            lsqnonlin(@GMM_multiP_moment_SARX,parmini,parm_lb,parm_ub,options,Y,L,Qhat,Phat,Wg(:,:,1),N,T,0);
        if option==4.2
            [Dt1,Sigmag_BGMM1,covmtx_BGMM1] = ...
                GMM_multiP_var_SARX_iid(parmhat_BGMM1,Y,L,Qhat,Phat,Wg(:,:,1),W,N,k,0);
        elseif option==5.2
            [Dt1,Sigmag_BGMM1,covmtx_BGMM1] = ...
                GMM_multiP_var_SARX_het(parmhat_BGMM1,Y,L,Qhat,Phat,Wg(:,:,1),W,N,k,0);
        end
            Wg(:,:,2) = pinv(Sigmag_BGMM1);
        if rho==0.95 && delta==1
            Wg(:,:,2) = nearestSPD(Wg(:,:,2));     
        end
        [parmhat_BGMM,resnorm,residual,exitflag,output] = ...
            lsqnonlin(@GMM_multiP_moment_SARX,parmhat_BGMM1,parm_lb,parm_ub,options,Y,L,Qhat,Phat,Wg(:,:,2),N,T,1);
        exitflagBIG(iter) = exitflag;
        eps_bound = 1e-3;
        if abs(parmhat_BGMM(1) - parm_lb(1))<eps_bound
           mark_boundary(iter)= -1;
           parmhat_BGMM(1) = -1+eps_bound;
        elseif abs(parmhat_BGMM(1)-parm_ub(1))<eps_bound
           mark_boundary(iter)= 1;
           parmhat_BGMM(1) = 1-eps_bound;
        end  
        parmhatBIG(iter,1:k+1) = parmhat_BGMM;
        
        if option==4.2
            Sigma1b = zeros(k+1,k+1);
            Sstar = speye(N)-parmhat_BGMM(1)*W;
            Gstar = W/Sstar;            
            Pstar = Gstar-trace(Gstar)/N*eye(N);
            Pstars = Pstar+Pstar';
            estarN = Y-L*parmhat_BGMM;
            sige2star = estarN'*estarN/N;
            GXb = Gstar*X_winpt*parmhat_BGMM(2:k+1);
            Sigma1b(1,1) = sum(sum(Gstar.*Pstars)) + GXb'*GXb/sige2star;
            Sigma1b(1,2:k+1) = GXb'* X_winpt/sige2star;
            Sigma1b(2:k+1,1) = Sigma1b(1,2:k+1);
            Sigma1b(2:k+1,2:k+1) = X_winpt'*X_winpt/sige2star;
            Sigma1b = Sigma1b./N;
            covmtx_BGMM = pinv(Sigma1b)./N;
        elseif option==5.2
             [Dt,Sigmag_BGMM,covmtx_BGMM] = ...
                 GMM_multiP_var_SARX_het(parmhat_BGMM,Y,L,Qhat,Phat,Wg(:,:,2),W,N,k,0);
        end
        seBIG(iter,1:k+1) = sqrt(diag(covmtx_BGMM));
        
    elseif flag_SARX==0
        clear options;
        X_winpt = [ones(N,1),X];
        L = [Ystar,X_winpt]; 
        parmini = parmhatBIG_GMM(iter,1:k+1)';
        Shat = speye(N)-parmini(1)*W;
        Ghat = W/Shat;
        if option==4.2
            Phat = Ghat-trace(Ghat)/N*eye(N);
        elseif option==5.2
            Phat = Ghat - diag(diag(Ghat));    
        end
        Qhat = X_winpt;
        q = size(Qhat,2);
        Wg(:,:,1) = eye(q+1);
        options = optimoptions(@lsqnonlin,'Algorithm','trust-region-reflective','Display','off');
        options.MaxFunEvals = 1e6;
        options.MaxIter = 1e6;
        options.TolFun = 1e-6;
        Mb = 1000;                  
        parm_lb = [-1;-Mb*ones(k,1)]; 
        parm_ub = [1;Mb*ones(k,1)];    
        [parmhat_BGMM,resnorm,residual,exitflag,output] = ...
            lsqnonlin(@GMM_multiP_moment_SARX,parmini,parm_lb,parm_ub,options,Y,L,Qhat,Phat,Wg(:,:,1),N,T,0);
        exitflagBIG(iter) = exitflag;
        eps_bound = 1e-3;
        if abs(parmhat_BGMM(1) - parm_lb(1))<eps_bound
           mark_boundary(iter)= -1;
           parmhat_BGMM(1) = -1+eps_bound;
        elseif abs(parmhat_BGMM(1)-parm_ub(1))<eps_bound
           mark_boundary(iter)= 1;
           parmhat_BGMM(1) = 1-eps_bound;
        end  
        parmhatBIG(iter,1:k+1) = parmhat_BGMM;

        Sigma1b = zeros(k+1,k+1);
        Sstar = speye(N)-parmhat_BGMM(1)*W;
        Gstar = W/Sstar;
        if option==4.2
            Pstar = Gstar-trace(Gstar)/N*eye(N);
        elseif option==5.2
            Pstar = Gstar-diag(diag(Gstar));
        end
        Pstars = Pstar+Pstar';
        estarN = Y-L*parmhat_BGMM;
        sige2star = estarN'*estarN/N;
        GXb = Gstar*X_winpt*parmhat_BGMM(2:k+1);
        Sigma1b(1,1) = sum(sum(Gstar.*Pstars)) + GXb'*GXb/sige2star;
        Sigma1b(1,2:k+1) = GXb'* X_winpt/sige2star;
        Sigma1b(2:k+1,1) = Sigma1b(1,2:k+1);
        Sigma1b(2:k+1,2:k+1) = X_winpt'*X_winpt/sige2star;
        Sigma1b = Sigma1b./N;
        covmtx_BGMM = pinv(Sigma1b)./N;
        seBIG(iter,1:k+1) = sqrt(diag(covmtx_BGMM));
    end

end
end    
%% OUTPUT
if flag_HetVar==0
    parm = [rho,betavec',sige2];
    epsilon = [0.1, 0.2*ones(1,k), 0.02];
    if option==3.1 || round(option)==5
        parm = parm(1:end-1);
        epsilon = epsilon(1:end-1);
    end
elseif flag_HetVar==1
    parm = [rho,betavec'];
    epsilon = [0.1, 0.2*ones(1,k)];
    if option==1 || option==3 || option==4
        parmhatBIG = parmhatBIG(:,1:end-1);
        seBIG = seBIG(:,1:end-1);
        parmhatBIG_sig2_no_use = parmhatBIG(:,end); % saved but no use
        seBIG_sig2_no_use = seBIG(:,end);
    end
end
% Bias
bias = mean(parmhatBIG) - parm;
% RMSE
dev = parmhatBIG - repmat(parm,nsimul,1);
mse = mean(dev.*dev);
rmse = sqrt(mse);
% Size
zsim = dev./seBIG;
siz = mean(abs(zsim)>1.96);
% Power
parm1 = parm - epsilon;
dev1 = parmhatBIG - repmat(parm1,nsimul,1);
z1sim = dev1./seBIG;
pow = mean(abs(z1sim)>1.96);
results = [bias;rmse;siz;pow];
results*100

% count boundary solutions
num_boundary = sum(abs(mark_boundary));
% if num_boundary~=0
%    warning('   boundary solution!');
%    num_boundary
% end

if nsimul==2000   % save results if number of replications==2000
     % need to manually create subfolders if saving results for the first time
    if flag_expodecay==0
        if num_domi==1
            if flag_SARX==1
                    fpath =[fdir,'\results_SARX_',date_string,'\']; 
                    fname =  [fpath, 'results_',date_string,'_',model_string,'_',err_string,'_',num2str(option),'_',...
                        num2str(rho),'_',num2str(N),'_',num2str(delta),'.mat'];
                if flag_Wx==1
                    fpath = [fdir,'\results_Wx_',date_string,'\'];
                    fname = [fpath,'results_',date_string,'_',model_string,'_',err_string,'_',num2str(option),'_',...
                        num2str(rho),'_',num2str(N),'_',num2str(delta),'_Wx_',num2str(deltax),'.mat'];                  
                end                
            elseif flag_SARX==0
                fpath = [fdir,'\results_SAR_',date_string,'\'];
                fname =  [fpath,'results_',date_string,'_',model_string,'_',err_string,'_',num2str(option),'_',...
                    num2str(rho),'_',num2str(N),'_',num2str(delta),'.mat'];
            end
        elseif num_domi==2
                fpath =[fdir,'\results_SARX_2domi',date_string,'\'];
                fname =  [fpath,'results_',date_string,'_',model_string,'_',err_string,'_',num2str(option),'_',...
                    num2str(rho),'_',num2str(N),'_',num2str(delta),'.mat'];
        end
    elseif flag_expodecay==1
        fpath = [fdir,'\results_SARX_expodecay_',date_string,'\'];
        fname = [fpath,'results_',date_string,'_',model_string,'_',err_string,'_',num2str(option),'_',...
            num2str(rho),'_',num2str(N),'_expodecay_',num2str(q_decay),'.mat'];
    end
    save(fname);  % comment out if do not want to save results
    
elseif nsimul==2  % only used to save Rsq tables
    Rsq_tab = permute(RsqBIG,[1 3 2]);
    Rsq_tab = reshape(Rsq_tab,[],size(RsqBIG,2),1);
    if flag_SARX==0         
         fpath = [fdir,'\results_SAR_',date_string,'\'];             
    elseif flag_SARX==1
         fpath = [fdir,'\results_SARX_',date_string,'\'];
    end
    fname = [fpath,'R2_',date_string,'_',model_string,'_',err_string,'.csv'];     
    csvwrite(fname,Rsq_tab,0,0);
    save([fpath,'R2_',date_string,'_',model_string,'.mat'],'RsqBIG');
    if flag_SARX==1      
        sigv2_tab = permute(sigv2BIG,[1 3 2]);   % error variance of X process
        sigv2_tab = reshape(sigv2_tab,[],size(sigv2BIG,2),1);
        csvwrite([fpath,'sigv2_',date_string,'_',model_string,'_',err_string,'.csv'],sigv2_tab,0,0);
        save([fpath,'sigv2_',date_string,'_',model_string,'.mat'],'sigv2BIG');
    end
    
end

    end
end
end  % end loop over rhoset
